#ifndef __MATH_HPP__
#define __MATH_HPP__

// globalna wartość PI
const double PI = 3.141592;
const double T_PI = PI * 2.0;
const double EPSILON = 0.001;

/**
 * @struct position2d
 * @brief pozycja dwuwymiarowa
 */
struct position2d
{
    /**
     * @brief domyslny konstr.
     * inicjalizuje wsp. wartosciami zerowymi
     */
    position2d( void ) : x( 0 ), y( 0 ) { }

    /// wspolrzedna x
    int x;

    /// wspolrzedna y
    int y;

} mousePosition;

/**
 * @struct vector3f
 * @brief Struktura przechowująca współrzędne wektora
 */
struct vector3f
{

    /**
     * @brief Domyslny konstruktor
     * Ustawia wszystkie wspolrzedne na ( 0, 0, 0 )
     */
    vector3f( void ) : x( .0f ), y( .0f ), z( .0f )
    {
    }

    /**
     * @brief Konstruktor ustawiajacy wspolrzedne
     * @param in_x wspolrzedna x
     * @param in_y wspolrzedna y
     * @param in_z wspolrzedna z
     */
    vector3f( GLfloat in_x, GLfloat in_y, GLfloat in_z )
        : x( in_x ), y( in_y ), z( in_z )
    {
    }

    /**
     * @brief normalizacja wektora
     * @return znormalizowany wektor
     */
    vector3f normalize( void ) const
    {
        GLfloat len = length();
        assert( len > 0 );

        GLfloat one_div_len = 1.0f / len;

        return vector3f(
            x * one_div_len,
            y * one_div_len,
            z * one_div_len );
    }

    /**
     * @brief liczenie dlugosci wektora
     * @return dlugosc wektora
     */
    GLfloat length( void ) const
    {
        return sqrtf( x * x + y * y + z * z );
    }

    ///wspolrzedna x
    GLfloat x;
    ///wspolrzedna y
    GLfloat y;
    ///wspolrzedna z
    GLfloat z;
};

/**
 * @brief Funkcja pomocnicza, przelicza pozycję z jednej przestrzeni na inna
 *        okreslona podanymi parametrami
 *
 * @param posX pozycja wejściowa
 * @param maxPosX maksymalna wartosc jaką może uzyskać parametr posX
 * @param dstPosMin minimalna wartosc pozycji w docelowej przestrzeni
 * @param dstPosMax maksymalna wartosc pozycji w docelowej przestrzeni
 */
GLfloat transformPos( GLfloat posX, GLfloat maxPosX,
							 GLfloat dstPosMin, GLfloat dstPosMax )
{
    GLfloat normalized = posX / maxPosX;
    return dstPosMin + ( ( dstPosMax - dstPosMin ) * normalized );
}


#endif // __MATH_HPP__
