#ifndef __RENDERER_HPP__
#define __RENDERER_HPP__

#include <GL/gl.h>
#include <GL/glu.h>

/**
 * @brief ustawia zrodlo swiatla punktowego
 * @param id numer swiatla
 * @param position pozycja światła w przestrzeni
 * @param ambient_col kolor światła otoczenia
 * @param diffuse_col kolor światła rozproszonego
 * @param specular_col kolor światła odbitego
 */
void setPointLight(
    GLenum id,
    GLfloat position[ 4 ],
    GLfloat ambient_col[ 4 ],
    GLfloat diffuse_col[ 4 ],
    GLfloat specular_col[ 4 ] )
{
    glEnable( id );

    glLightfv( id, GL_AMBIENT, ambient_col );
    glLightfv( id, GL_DIFFUSE, diffuse_col );
    glLightfv( id, GL_SPECULAR, specular_col );
    glLightfv( id, GL_POSITION, position );
}

/**
 * @brief funkcja ustawia parametry materiału
 * @param ambient_col kolor materiału nieoświetlonego żadnym światłem
 * @param diffuse_col w jaki sposób materiał odbija światło rozproszone
 * @param specular_col w jaki sposób materiał odbija światło odbite
 * @param shiness połyskliwość materiału
 */
void setMaterial(
    GLfloat ambient_col[ 4 ],
    GLfloat diffuse_col[ 4 ],
    GLfloat specular_col[ 4 ],
    GLfloat shiness )
{
    glMaterialfv( GL_FRONT, GL_AMBIENT, ambient_col );
    glMaterialfv( GL_FRONT, GL_DIFFUSE, diffuse_col );
    glMaterialfv( GL_FRONT, GL_SPECULAR, specular_col );

    glMaterialf( GL_FRONT, GL_SHININESS, shiness );
}

/**
 * @brief Iniclalizacja stanu poczatkowego niektorych
 *         wartosci OpenGL.
 * @param width szerokosc ekranu
 * @param height wysokosc ekranu
 */
void initOpenGL( int width, int height )
{
    glMatrixMode( GL_PROJECTION );
    gluPerspective( 45.0f, width / ( GLfloat ) height, 0.1f, 100.0f );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    //włączenie testowania bufora głębokości
    glEnable( GL_DEPTH_TEST );

    //sposób rasteryzacji wielokątów
    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

    //włączenie liczenia oświetlenia
    glDisable( GL_LIGHTING );
}

#endif // __RENDERER_HPP__
